import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { IServiceProvider } from "./IServiceProvider.js";
import { ServiceLifetime } from "./ServiceLifetime.js";
import { ServiceRegistry } from "./ServiceRegistry.js";
import { isServiceImplementationResolver } from "../implementation-resolvers/BaseServiceImplementationResolver.js";
import { RegistryServiceImplementationResolver } from "../implementation-resolvers/RegistryServiceImplementationResolver.js";
import { ValueServiceImplementationResolver } from "../implementation-resolvers/ValueServiceImplementationResolver.js";
export class ServiceCollection {
    collection;
    constructor(collection) {
        this.collection = new Map(collection?.copy() || []);
        if (!this.collection.has(IServiceProvider)) {
            this.addTransient(IServiceProvider);
        }
    }
    *[Symbol.iterator]() {
        for (const descriptors of this.collection.values()) {
            for (const descriptor of descriptors.all) {
                yield descriptor;
            }
        }
    }
    getDescriptors(service) {
        return (this.collection.get(service)?.all ?? []);
    }
    getDescriptor(service) {
        const descriptor = this.collection.get(service);
        if (!descriptor) {
            let name = getNameOfServiceType(service);
            const metadata = ServiceRegistry.get(service);
            if (metadata?.name) {
                name = metadata.name;
            }
            throw new Error(`No service of type "${name}" is registered`);
        }
        return descriptor.single;
    }
    addScoped(service, implementation = service) {
        this.addService(ServiceLifetime.Scoped, service, implementation);
        return this;
    }
    addTransient(service, implementation = service) {
        this.addService(ServiceLifetime.Transient, service, implementation);
        return this;
    }
    addSingleton(service, implementation = service) {
        this.addService(ServiceLifetime.Singleton, service, implementation);
        return this;
    }
    *copy() {
        for (const [key, descriptors] of this.collection) {
            const all = [...descriptors.all.map((d) => ({ ...d }))];
            yield [
                key,
                {
                    single: all[all.length - 1],
                    all,
                },
            ];
        }
    }
    addService(lifetime, service, implementation) {
        let resolver;
        if (isServiceImplementationResolver(implementation)) {
            resolver = implementation;
        }
        else if (typeof implementation === "function") {
            resolver = new RegistryServiceImplementationResolver(implementation);
        }
        else {
            resolver = new ValueServiceImplementationResolver(implementation);
        }
        const descriptor = {
            lifetime,
            service,
            resolver,
        };
        let descriptors = this.collection.get(service);
        if (!descriptors) {
            this.collection.set(service, {
                single: descriptor,
                all: [descriptor],
            });
        }
        else {
            descriptors.all.push(descriptor);
            descriptors.single = descriptor;
        }
        return this;
    }
}
//# sourceMappingURL=ServiceCollection.js.map