import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
const NOT_INITIALIZED = Symbol("NOT_INITIALIZED");
export class ServiceInstanceInfo {
    descriptor;
    get instance() {
        if (!this.initialized) {
            throw new Error(`Service "${getNameOfDescriptor(this.descriptor)}" is not initialized (circular dependency)`);
        }
        return this._instance;
    }
    get initialized() {
        return this._instance !== NOT_INITIALIZED;
    }
    disposed;
    dependents;
    _instance;
    constructor(descriptor) {
        this.descriptor = descriptor;
        this.disposed = false;
        this.dependents = [];
        this._instance = NOT_INITIALIZED;
    }
    *getInstance() {
        return this._instance;
    }
    initialize(creator) {
        if (this._instance === NOT_INITIALIZED) {
            this._instance = creator();
        }
        return this._instance;
    }
    addDependent(dependent) {
        this.dependents.push(dependent);
    }
    disposeSync() {
        if (this.disposed || !this.initialized) {
            return;
        }
        const instance = this._instance;
        const disposeMethod = instance?.[Symbol.dispose] ?? instance?.dispose;
        if (typeof disposeMethod === "function") {
            for (const dependent of this.dependents) {
                dependent.disposeSync();
            }
            Reflect.apply(disposeMethod, instance, []);
            this.disposed = true;
        }
    }
    async disposeAsync() {
        if (this.disposed || !this.initialized) {
            return;
        }
        const instance = this._instance;
        const disposeMethod = instance?.[Symbol.asyncDispose] ?? instance?.dispose;
        if (typeof disposeMethod === "function") {
            this.disposed = true;
            try {
                await Promise.all([...this.dependents].map((dependent) => dependent.disposeAsync()));
                await Reflect.apply(disposeMethod, instance, []);
            }
            catch (e) {
                this.disposed = false;
                throw e;
            }
        }
        else {
            this.disposeSync();
        }
    }
}
//# sourceMappingURL=ServiceInstanceInfo.js.map