import { IServiceProvider } from "./IServiceProvider.js";
import { ServiceCollection } from "./ServiceCollection.js";
import { ServiceInstancesStore } from "./ServiceInstancesStore.js";
import { ServiceLifetime } from "./ServiceLifetime.js";
import { validateRequestPath } from "./validation/validateRequestPath.js";
import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
import { resolveGeneratorAsync } from "../helpers/resolveGeneratorAsync.js";
import { resolveGeneratorSync } from "../helpers/resolveGeneratorSync.js";
import { isServiceProvider } from "../helpers/isServiceProvider.js";
import { all } from "../service-type-resolvers/all.js";
import { single } from "../service-type-resolvers/single.js";
import { isServiceTypeResolver } from "../service-type-resolvers/BaseServiceTypeResolver.js";
import { Debug } from "../debug.js";
const EMPTY_PATH = {
    value: null,
    next: null,
};
export class ServiceProvider {
    collection;
    parent;
    static internalGetService(provider, service, requestedBy, requestedPath, mode) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.internalGetService(service, requestedBy, requestedPath, mode);
    }
    static getDescriptor(provider, service) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.collection.getDescriptor(service);
    }
    static getDescriptors(provider, service) {
        if (!(provider instanceof ServiceProvider)) {
            throw new Error("provider must be an instance of ServiceProvider");
        }
        return provider.collection.getDescriptors(service);
    }
    instancesStore;
    constructor(collection, parent) {
        this.collection = collection;
        this.parent = parent;
        this.instancesStore = new ServiceInstancesStore();
        this.internalGetService = this.internalGetService.bind(this);
        this.resolveServiceImplementation =
            this.resolveServiceImplementation.bind(this);
    }
    getServices(service) {
        return resolveGeneratorSync(this.internalGetService(all(service), null, EMPTY_PATH, "sync"));
    }
    getServiceAsync(service) {
        if (!isServiceTypeResolver(service)) {
            service = single(service);
        }
        return resolveGeneratorAsync(this.internalGetService(service, null, EMPTY_PATH, "async"));
    }
    getService(service) {
        if (!isServiceTypeResolver(service)) {
            service = single(service);
        }
        return resolveGeneratorSync(this.internalGetService(service, null, EMPTY_PATH, "sync"));
    }
    getServicesAsync(service) {
        return resolveGeneratorAsync(this.internalGetService(all(service), null, EMPTY_PATH, "async"));
    }
    createScope() {
        const serviceProvider = new ServiceProvider(this.collection, this);
        return {
            serviceProvider,
            [Symbol.dispose]: () => {
                serviceProvider[Symbol.dispose]();
            },
        };
    }
    createAsyncScope() {
        const serviceProvider = new ServiceProvider(this.collection, this);
        return {
            serviceProvider,
            [Symbol.asyncDispose]: async () => {
                await serviceProvider[Symbol.asyncDispose]();
            },
        };
    }
    internalGetService(service, requestedBy, requestedPath, mode) {
        return service.resolve(this.collection, this.instancesStore, this.resolveServiceImplementation, requestedBy, requestedPath, mode);
    }
    resolveServiceImplementation(descriptor, requestedBy, requestedPath, mode) {
        if (descriptor.lifetime === ServiceLifetime.Singleton && this.parent) {
            return this.parent.resolveServiceImplementation(descriptor, requestedBy, requestedPath, mode);
        }
        if (descriptor.lifetime !== ServiceLifetime.Transient) {
            const instanceInfo = this.instancesStore.getInstanceInfo(descriptor);
            if (instanceInfo?.initialized) {
                return instanceInfo.getInstance();
            }
        }
        if (Debug.extended) {
            validateRequestPath(requestedPath, descriptor);
        }
        return this.createInstanceFromDescriptor(descriptor, requestedBy, requestedPath, mode);
    }
    *createInstanceFromDescriptor(descriptor, requestedBy, requestedPath, mode) {
        try {
            if (descriptor.lifetime === ServiceLifetime.Transient) {
                if (isServiceProvider(descriptor.service)) {
                    return this;
                }
                return (yield* descriptor.resolver.resolve(this.internalGetService, descriptor, requestedBy, requestedPath, mode))();
            }
            if (descriptor.lifetime === ServiceLifetime.Scoped) {
                if (requestedBy?.lifetime === ServiceLifetime.Singleton) {
                    throw new Error(`Scoped service cannot be resolved from singleton service.`);
                }
                if (!this.parent) {
                    throw new Error("Scoped services require a service scope.");
                }
            }
            return this.instancesStore
                .addInstance(descriptor, requestedBy)
                .initialize(yield* descriptor.resolver.resolve(this.internalGetService, descriptor, requestedBy, requestedPath, mode));
        }
        catch (exception) {
            throw new Error(`Failed to initiate service ${getNameOfDescriptor(descriptor)}:\n\r${exception.message}`, { cause: exception });
        }
    }
    [Symbol.dispose]() {
        this.instancesStore[Symbol.dispose]();
    }
    async [Symbol.asyncDispose]() {
        await this.instancesStore[Symbol.asyncDispose]();
    }
}
//# sourceMappingURL=ServiceProvider.js.map